@@ -42,7 +42,7 @@ struct _SpCaptureReader
   gint64               end_time;
 };
 
-#ifndef SP_DISABLE_GOBJECT
+#ifdef SP_ENABLE_GOBJECT
 G_DEFINE_BOXED_TYPE (SpCaptureReader, sp_capture_reader,
                      sp_capture_reader_ref, sp_capture_reader_unref)
 #endif
@@ -106,6 +106,17 @@ sp_capture_reader_get_filename (SpCaptureReader *self)
   return self->filename;
 }
 
+/**
+ * sp_capture_reader_new_from_fd:
+ * @fd: an fd to take ownership from
+ * @error: a location for a #GError or %NULL
+ *
+ * Creates a new reader using the file-descriptor.
+ *
+ * This is useful if you don't necessarily have access to the filename itself.
+ *
+ * Returns: (transfer full): an #SpCaptureReader or %NULL upon failure.
+ */
 SpCaptureReader *
 sp_capture_reader_new_from_fd (int      fd,
                                GError **error)
@@ -198,6 +209,17 @@ sp_capture_reader_bswap_map (SpCaptureReader *self,
     }
 }
 
+static inline void
+sp_capture_reader_bswap_mark (SpCaptureReader *self,
+                              SpCaptureMark   *mark)
+{
+  g_assert (self != NULL);
+  g_assert (mark != NULL);
+
+  if (G_UNLIKELY (self->endian != G_BYTE_ORDER))
+    mark->duration = GUINT64_SWAP_LE_BE (mark->duration);
+}
+
 static inline void
 sp_capture_reader_bswap_jitmap (SpCaptureReader *self,
                                 SpCaptureJitmap *jitmap)
@@ -426,6 +448,47 @@ sp_capture_reader_read_map (SpCaptureReader *self)
   return map;
 }
 
+const SpCaptureMark *
+sp_capture_reader_read_mark (SpCaptureReader *self)
+{
+  SpCaptureMark *mark;
+
+  g_assert (self != NULL);
+  g_assert ((self->pos % SP_CAPTURE_ALIGN) == 0);
+  g_assert (self->pos <= self->bufsz);
+
+  if (!sp_capture_reader_ensure_space_for (self, sizeof *mark))
+    return NULL;
+
+  mark = (SpCaptureMark *)(gpointer)&self->buf[self->pos];
+
+  sp_capture_reader_bswap_frame (self, &mark->frame);
+
+  if (mark->frame.type != SP_CAPTURE_FRAME_MARK)
+    return NULL;
+
+  if (mark->frame.len < (sizeof *mark + 1))
+    return NULL;
+
+  if (!sp_capture_reader_ensure_space_for (self, mark->frame.len))
+    return NULL;
+
+  mark = (SpCaptureMark *)(gpointer)&self->buf[self->pos];
+
+  sp_capture_reader_bswap_mark (self, mark);
+
+  self->pos += mark->frame.len;
+
+  if ((self->pos % SP_CAPTURE_ALIGN) != 0)
+    return NULL;
+
+  /* Ensure trailing \0 in name and message */
+  mark->name[sizeof mark->name - 1] = 0;
+  self->buf[self->pos + mark->frame.len - 1] = 0;
+
+  return mark;
+}
+
 const SpCaptureProcess *
 sp_capture_reader_read_process (SpCaptureReader *self)
 {
