@@ -34,6 +34,7 @@
 
 #define DEFAULT_BUFFER_SIZE (getpagesize() * 64L)
 #define INVALID_ADDRESS     (G_GUINT64_CONSTANT(0))
+#define MAX_COUNTERS        ((1 << 24) - 1)
 
 typedef struct
 {
@@ -96,7 +97,7 @@ struct _SpCaptureWriter
   SpCaptureStat stat;
 };
 
-#ifndef SP_DISABLE_GOBJECT
+#ifdef SP_ENABLE_GOBJECT
 G_DEFINE_BOXED_TYPE (SpCaptureWriter, sp_capture_writer,
                      sp_capture_writer_ref, sp_capture_writer_unref)
 #endif
@@ -536,6 +537,50 @@ sp_capture_writer_add_map (SpCaptureWriter *self,
   return TRUE;
 }
 
+gboolean
+sp_capture_writer_add_mark (SpCaptureWriter *self,
+                            gint64           time,
+                            gint             cpu,
+                            GPid             pid,
+                            guint64          duration,
+                            const gchar     *group,
+                            const gchar     *name,
+                            const gchar     *message)
+{
+  SpCaptureMark *ev;
+  gsize message_len;
+  gsize len;
+
+  g_assert (self != NULL);
+  g_assert (name != NULL);
+  g_assert (group != NULL);
+
+  if (message == NULL)
+    message = "";
+  message_len = strlen (message) + 1;
+
+  len = sizeof *ev + message_len;
+  ev = (SpCaptureMark *)sp_capture_writer_allocate (self, &len);
+  if (!ev)
+    return FALSE;
+
+  sp_capture_writer_frame_init (&ev->frame,
+                                len,
+                                cpu,
+                                pid,
+                                time,
+                                SP_CAPTURE_FRAME_MARK);
+
+  ev->duration = duration;
+  g_strlcpy (ev->group, group, sizeof ev->group);
+  g_strlcpy (ev->name, name, sizeof ev->name);
+  memcpy (ev->message, message, message_len);
+
+  self->stat.frame_count[SP_CAPTURE_FRAME_MARK]++;
+
+  return TRUE;
+}
+
 SpCaptureAddress
 sp_capture_writer_add_jitmap (SpCaptureWriter *self,
                               const gchar     *name)
@@ -1035,7 +1080,15 @@ sp_capture_writer_define_counters (SpCaptureWriter        *self,
   def->n_counters = n_counters;
 
   for (i = 0; i < n_counters; i++)
-    def->counters[i] = counters[i];
+    {
+      if (counters[i].id >= self->next_counter_id)
+        {
+          g_warning ("Counter %u has not been registered.", counters[i].id);
+          continue;
+        }
+
+      def->counters[i] = counters[i];
+    }
 
   self->stat.frame_count[SP_CAPTURE_FRAME_CTRDEF]++;
 
@@ -1076,6 +1129,8 @@ sp_capture_writer_set_counters (SpCaptureWriter             *self,
   if (!set)
     return FALSE;
 
+  memset (set, 0, len);
+
   sp_capture_writer_frame_init (&set->frame,
                                 len,
                                 cpu,
@@ -1104,7 +1159,20 @@ sp_capture_writer_set_counters (SpCaptureWriter             *self,
   return TRUE;
 }
 
-gint
+/**
+ * sp_capture_writer_request_counter:
+ *
+ * This requests a series of counter identifiers for the capture.
+ *
+ * The returning number is always greater than zero. The resulting counter
+ * values are monotonic starting from the resulting value.
+ *
+ * For example, if you are returned 5, and requested 3 counters, the counter
+ * ids you should use are 5, 6, and 7.
+ *
+ * Returns: The next series of counter values or zero on failure.
+ */
+guint
 sp_capture_writer_request_counter (SpCaptureWriter *self,
                                    guint            n_counters)
 {
@@ -1112,8 +1180,41 @@ sp_capture_writer_request_counter (SpCaptureWriter *self,
 
   g_assert (self != NULL);
 
+  if (MAX_COUNTERS - n_counters < self->next_counter_id)
+    return 0;
+
   ret = self->next_counter_id;
   self->next_counter_id += n_counters;
 
   return ret;
 }
+
+gboolean
+_sp_capture_writer_set_time_range (SpCaptureWriter *self,
+                                   gint64           start_time,
+                                   gint64           end_time)
+{
+  ssize_t ret;
+
+  g_assert (self != NULL);
+
+do_start:
+  ret = pwrite (self->fd,
+                &start_time,
+                sizeof (start_time),
+                G_STRUCT_OFFSET (SpCaptureFileHeader, time));
+
+  if (ret < 0 && errno == EAGAIN)
+    goto do_start;
+
+do_end:
+  ret = pwrite (self->fd,
+                &end_time,
+                sizeof (end_time),
+                G_STRUCT_OFFSET (SpCaptureFileHeader, end_time));
+
+  if (ret < 0 && errno == EAGAIN)
+    goto do_end;
+
+  return TRUE;
+}
