@@ -69,8 +69,8 @@ typedef struct
 {
   guint id;
   gdouble line_width;
-  GdkRGBA background;
   GdkRGBA foreground;
+  GdkRGBA background;
   guint use_default_style : 1;
   guint fill : 1;
 } LineInfo;
@@ -170,8 +170,8 @@ sp_line_visualizer_row_draw (GtkWidget *widget,
 
       if (n_fpoints > 0)
         {
-          gfloat last_x;
-          gfloat last_y;
+          gdouble last_x;
+          gdouble last_y;
 
           points = g_new0 (SpVisualizerRowAbsolutePoint, n_fpoints);
 
@@ -215,17 +215,19 @@ sp_line_visualizer_row_draw (GtkWidget *widget,
 
           cairo_set_line_width (cr, line_info->line_width);
 
+          if (line_info->fill)
+            {
+              gdk_cairo_set_source_rgba (cr, &line_info->background);
+              cairo_fill_preserve (cr);
+            }
+
           if (line_info->use_default_style)
             color = foreground;
           else
             color = line_info->foreground;
 
           gdk_cairo_set_source_rgba (cr, &color);
-
-          if (line_info->fill)
-            cairo_fill (cr);
-          else
-            cairo_stroke (cr);
+          cairo_stroke (cr);
         }
     }
 
@@ -481,6 +483,7 @@ sp_line_visualizer_row_add_counter (SpLineVisualizerRow *self,
     }
   else
     {
+      gdk_rgba_parse (&line_info.foreground, "#000");
       line_info.use_default_style = TRUE;
     }
 
@@ -531,7 +534,7 @@ counter_type (LoadData *load,
   return SP_CAPTURE_COUNTER_DOUBLE;
 }
 
-static inline gfloat
+static inline gdouble
 calc_x (gint64 lower,
         gint64 upper,
         gint64 value)
@@ -539,7 +542,7 @@ calc_x (gint64 lower,
   return (gdouble)(value - lower) / (gdouble)(upper - lower);
 }
 
-static inline gfloat
+static inline gdouble
 calc_y_double (gdouble lower,
                gdouble upper,
                gdouble value)
@@ -547,7 +550,7 @@ calc_y_double (gdouble lower,
   return (value - lower) / (upper - lower);
 }
 
-static inline gfloat
+static inline gdouble
 calc_y_int64 (gint64 lower,
               gint64 upper,
               gint64 value)
@@ -569,7 +572,7 @@ sp_line_visualizer_row_load_data_frame_cb (const SpCaptureFrame *frame,
   if (frame->type == SP_CAPTURE_FRAME_CTRSET)
     {
       const SpCaptureFrameCounterSet *set = (SpCaptureFrameCounterSet *)frame;
-      gfloat x = calc_x (load->begin_time, load->end_time, frame->time);
+      gdouble x = calc_x (load->begin_time, load->end_time, frame->time);
 
       for (guint i = 0; i < set->n_values; i++)
         {
@@ -581,7 +584,7 @@ sp_line_visualizer_row_load_data_frame_cb (const SpCaptureFrame *frame,
 
               if (counter_id != 0 && contains_id (load->lines, counter_id))
                 {
-                  gfloat y;
+                  gdouble y;
 
                   if (counter_type (load, counter_id) == SP_CAPTURE_COUNTER_DOUBLE)
                     y = calc_y_double (load->y_lower, load->y_upper, group->values[j].vdbl);
@@ -682,3 +685,49 @@ sp_line_visualizer_row_load_data_finish (SpLineVisualizerRow  *self,
 
   return g_task_propagate_pointer (G_TASK (result), error);
 }
+
+void
+sp_line_visualizer_row_set_line_width (SpLineVisualizerRow *self,
+                                       guint                counter_id,
+                                       gdouble              width)
+{
+  SpLineVisualizerRowPrivate *priv = sp_line_visualizer_row_get_instance_private (self);
+
+  g_return_if_fail (SP_IS_LINE_VISUALIZER_ROW (self));
+
+  for (guint i = 0; i < priv->lines->len; i++)
+    {
+      LineInfo *info = &g_array_index (priv->lines, LineInfo, i);
+
+      if (info->id == counter_id)
+        {
+          info->line_width = width;
+          sp_line_visualizer_row_queue_reload (self);
+          break;
+        }
+    }
+}
+
+void
+sp_line_visualizer_row_set_fill (SpLineVisualizerRow *self,
+                                 guint                counter_id,
+                                 const GdkRGBA       *color)
+{
+  SpLineVisualizerRowPrivate *priv = sp_line_visualizer_row_get_instance_private (self);
+
+  g_return_if_fail (SP_IS_LINE_VISUALIZER_ROW (self));
+
+  for (guint i = 0; i < priv->lines->len; i++)
+    {
+      LineInfo *info = &g_array_index (priv->lines, LineInfo, i);
+
+      if (info->id == counter_id)
+        {
+          info->fill = !!color;
+          if (color != NULL)
+            info->background = *color;
+          sp_line_visualizer_row_queue_reload (self);
+          break;
+        }
+    }
+}
