@@ -190,6 +190,7 @@ test_reader_basic (void)
 
   {
     SpCaptureCounter counters[10];
+    guint base = sp_capture_writer_request_counter (writer, G_N_ELEMENTS (counters));
 
     t = SP_CAPTURE_CURRENT_TIME;
 
@@ -198,7 +199,7 @@ test_reader_basic (void)
         g_snprintf (counters[i].category, sizeof counters[i].category, "cat%d", i);
         g_snprintf (counters[i].name, sizeof counters[i].name, "name%d", i);
         g_snprintf (counters[i].description, sizeof counters[i].description, "desc%d", i);
-        counters[i].id = i + 1;
+        counters[i].id = base + i;
         counters[i].type = 0;
         counters[i].value.v64 = i * G_GINT64_CONSTANT (100000000000);
       }
@@ -531,6 +532,53 @@ test_reader_splice (void)
   g_unlink ("writer3.syscap");
 }
 
+static void
+test_reader_writer_mark (void)
+{
+  SpCaptureWriter *writer;
+  SpCaptureReader *reader;
+  const SpCaptureMark *mark;
+  SpCaptureFrameType type;
+  GError *error = NULL;
+  gint r;
+
+  writer = sp_capture_writer_new ("mark1.syscap", 0);
+
+  sp_capture_writer_add_mark (writer, SP_CAPTURE_CURRENT_TIME, -1, -1, 125, "thread-0", "Draw", "hdmi-1");
+  sp_capture_writer_add_mark (writer, SP_CAPTURE_CURRENT_TIME, -1, -1, 0, "thread-1", "Deadline", "hdmi-1");
+
+  g_clear_pointer (&writer, sp_capture_writer_unref);
+
+  reader = sp_capture_reader_new ("mark1.syscap", &error);
+  g_assert_no_error (error);
+  g_assert (reader != NULL);
+
+  mark = sp_capture_reader_read_mark (reader);
+  g_assert_nonnull (mark);
+  g_assert_cmpstr (mark->group, ==, "thread-0");
+  g_assert_cmpstr (mark->name, ==, "Draw");
+  g_assert_cmpint (mark->duration, ==, 125);
+  g_assert_cmpstr (mark->message, ==, "hdmi-1");
+  g_assert_cmpint (mark->frame.time, >, 0);
+  g_assert_cmpint (mark->frame.cpu, ==, -1);
+
+  mark = sp_capture_reader_read_mark (reader);
+  g_assert_nonnull (mark);
+  g_assert_cmpstr (mark->group, ==, "thread-1");
+  g_assert_cmpstr (mark->name, ==, "Deadline");
+  g_assert_cmpint (mark->duration, ==, 0);
+  g_assert_cmpstr (mark->message, ==, "hdmi-1");
+  g_assert_cmpint (mark->frame.time, >, 0);
+  g_assert_cmpint (mark->frame.cpu, ==, -1);
+
+  r = sp_capture_reader_peek_type (reader, &type);
+  g_assert_cmpint (r, ==, FALSE);
+
+  g_clear_pointer (&reader, sp_capture_reader_unref);
+
+  g_unlink ("mark1.syscap");
+}
+
 int
 main (int argc,
       char *argv[])
@@ -540,5 +588,6 @@ main (int argc,
   g_test_add_func ("/SpCapture/ReaderWriter", test_reader_basic);
   g_test_add_func ("/SpCapture/Writer/splice", test_writer_splice);
   g_test_add_func ("/SpCapture/Reader/splice", test_reader_splice);
+  g_test_add_func ("/SpCapture/ReaderWriter/mark", test_reader_writer_mark);
   return g_test_run ();
 }
